let from_rgb (red, green, blue) =
  Gg.Color.v_srgb
    (float red /. 255.)
    (float green /. 255.)
    (float blue /. 255.)

module Pure =
struct
  let rec range low high =
    if low > high then []
    else low :: range (low+1) high

  let gradient ~n left right =
    if (n < 2) then
      raise (Invalid_argument "Color.gradient: needs at least two points")
    else
      range 0 (n-1) |> List.map @@ fun i ->
      let p = float i /. float (n-1) in 
      Gg.V4.( p * left + (1. -. p) * right)


  let aliceblue	= from_rgb (240, 248, 255)
  let antiquewhite	= from_rgb (250, 235, 215)
  let aqua	= from_rgb ( 0, 255, 255)
  let aquamarine	= from_rgb (127, 255, 212)
  let azure	= from_rgb (240, 255, 255)
  let beige	= from_rgb (245, 245, 220)
  let bisque	= from_rgb (255, 228, 196)
  let black	= from_rgb ( 0, 0, 0)
  let blanchedalmond	= from_rgb (255, 235, 205)
  let blue	= from_rgb ( 0, 0, 255)
  let blueviolet	= from_rgb (138, 43, 226)
  let brown	= from_rgb (165, 42, 42)
  let burlywood	= from_rgb (222, 184, 135)
  let cadetblue	= from_rgb ( 95, 158, 160)
  let chartreuse	= from_rgb (127, 255, 0)
  let chocolate	= from_rgb (210, 105, 30)
  let coral	= from_rgb (255, 127, 80)
  let cornflowerblue	= from_rgb (100, 149, 237)
  let cornsilk	= from_rgb (255, 248, 220)
  let crimson	= from_rgb (220, 20, 60)
  let cyan	= from_rgb ( 0, 255, 255)
  let darkblue	= from_rgb ( 0, 0, 139)
  let darkcyan	= from_rgb ( 0, 139, 139)
  let darkgoldenrod	= from_rgb (184, 134, 11)
  let darkgray	= from_rgb (169, 169, 169)
  let darkgreen	= from_rgb ( 0, 100, 0)
  let darkgrey	= from_rgb (169, 169, 169)
  let darkkhaki	= from_rgb (189, 183, 107)
  let darkmagenta	= from_rgb (139, 0, 139)
  let darkolivegreen	= from_rgb ( 85, 107, 47)
  let darkorange	= from_rgb (255, 140, 0)
  let darkorchid	= from_rgb (153, 50, 204)
  let darkred	= from_rgb (139, 0, 0)
  let darksalmon	= from_rgb (233, 150, 122)
  let darkseagreen	= from_rgb (143, 188, 143)
  let darkslateblue	= from_rgb ( 72, 61, 139)
  let darkslategray	= from_rgb ( 47, 79, 79)
  let darkslategrey	= from_rgb ( 47, 79, 79)
  let darkturquoise	= from_rgb ( 0, 206, 209)
  let darkviolet	= from_rgb (148, 0, 211)
  let deeppink	= from_rgb (255, 20, 147)
  let deepskyblue	= from_rgb ( 0, 191, 255)
  let dimgray	= from_rgb (105, 105, 105)
  let dimgrey	= from_rgb (105, 105, 105)
  let dodgerblue	= from_rgb ( 30, 144, 255)
  let firebrick	= from_rgb (178, 34, 34)
  let floralwhite	= from_rgb (255, 250, 240)
  let forestgreen	= from_rgb ( 34, 139, 34)
  let fuchsia	= from_rgb (255, 0, 255)
  let gainsboro	= from_rgb (220, 220, 220)
  let ghostwhite	= from_rgb (248, 248, 255)
  let gold	= from_rgb (255, 215, 0)
  let goldenrod	= from_rgb (218, 165, 32)
  let gray	= from_rgb (128, 128, 128)
  let grey	= from_rgb (128, 128, 128)
  let green	= from_rgb ( 0, 128, 0)
  let greenyellow	= from_rgb (173, 255, 47)
  let honeydew	= from_rgb (240, 255, 240)
  let hotpink	= from_rgb (255, 105, 180)
  let indianred	= from_rgb (205, 92, 92)
  let indigo	= from_rgb ( 75, 0, 130)
  let ivory	= from_rgb (255, 255, 240)
  let khaki	= from_rgb (240, 230, 140)
  let lavender	= from_rgb (230, 230, 250)
  let lavenderblush	= from_rgb (255, 240, 245)
  let lawngreen	= from_rgb (124, 252, 0)
  let lemonchiffon	= from_rgb (255, 250, 205)
  let lightblue	= from_rgb (173, 216, 230)
  let lightcoral	= from_rgb (240, 128, 128)
  let lightcyan	= from_rgb (224, 255, 255)
  let lightgoldenrodyellow	= from_rgb (250, 250, 210)
  let lightgray	= from_rgb (211, 211, 211)
  let lightgreen	= from_rgb (144, 238, 144)
  let lightgrey	= from_rgb (211, 211, 211)
  let lightpink	= from_rgb (255, 182, 193)
  let lightsalmon	= from_rgb (255, 160, 122)
  let lightseagreen	= from_rgb ( 32, 178, 170)
  let lightskyblue	= from_rgb (135, 206, 250)
  let lightslategray	= from_rgb (119, 136, 153)
  let lightslategrey	= from_rgb (119, 136, 153)
  let lightsteelblue	= from_rgb (176, 196, 222)
  let lightyellow	= from_rgb (255, 255, 224)
  let lime	= from_rgb ( 0, 255, 0)
  let limegreen	= from_rgb ( 50, 205, 50)
  let linen	= from_rgb (250, 240, 230)
  let magenta	= from_rgb (255, 0, 255)
  let maroon	= from_rgb (128, 0, 0)
  let mediumaquamarine	= from_rgb (102, 205, 170)
  let mediumblue	= from_rgb ( 0, 0, 205)
  let mediumorchid	= from_rgb (186, 85, 211)
  let mediumpurple	= from_rgb (147, 112, 219)
  let mediumseagreen	= from_rgb ( 60, 179, 113)
  let mediumslateblue	= from_rgb (123, 104, 238)
  let mediumspringgreen	= from_rgb ( 0, 250, 154)
  let mediumturquoise	= from_rgb ( 72, 209, 204)
  let mediumvioletred	= from_rgb (199, 21, 133)
  let midnightblue	= from_rgb ( 25, 25, 112)
  let mintcream	= from_rgb (245, 255, 250)
  let mistyrose	= from_rgb (255, 228, 225)
  let moccasin	= from_rgb (255, 228, 181)
  let navajowhite	= from_rgb (255, 222, 173)
  let navy	= from_rgb ( 0, 0, 128)
  let oldlace	= from_rgb (253, 245, 230)
  let olive	= from_rgb (128, 128, 0)
  let olivedrab	= from_rgb (107, 142, 35)
  let orange	= from_rgb (255, 165, 0)
  let orangered	= from_rgb (255, 69, 0)
  let orchid	= from_rgb (218, 112, 214)
  let palegoldenrod	= from_rgb (238, 232, 170)
  let palegreen	= from_rgb (152, 251, 152)
  let paleturquoise	= from_rgb (175, 238, 238)
  let palevioletred	= from_rgb (219, 112, 147)
  let papayawhip	= from_rgb (255, 239, 213)
  let peachpuff	= from_rgb (255, 218, 185)
  let peru	= from_rgb (205, 133, 63)
  let pink	= from_rgb (255, 192, 203)
  let plum	= from_rgb (221, 160, 221)
  let powderblue	= from_rgb (176, 224, 230)
  let purple	= from_rgb (128, 0, 128)
  let red	= from_rgb (255, 0, 0)
  let rosybrown	= from_rgb (188, 143, 143)
  let royalblue	= from_rgb ( 65, 105, 225)
  let saddlebrown	= from_rgb (139, 69, 19)
  let salmon	= from_rgb (250, 128, 114)
  let sandybrown	= from_rgb (244, 164, 96)
  let seagreen	= from_rgb ( 46, 139, 87)
  let seashell	= from_rgb (255, 245, 238)
  let sienna	= from_rgb (160, 82, 45)
  let silver	= from_rgb (192, 192, 192)
  let skyblue	= from_rgb (135, 206, 235)
  let slateblue	= from_rgb (106, 90, 205)
  let slategray	= from_rgb (112, 128, 144)
  let slategrey	= from_rgb (112, 128, 144)
  let snow	= from_rgb (255, 250, 250)
  let springgreen	= from_rgb ( 0, 255, 127)
  let steelblue	= from_rgb ( 70, 130, 180)
  let tan	= from_rgb (210, 180, 140)
  let teal	= from_rgb ( 0, 128, 128)
  let thistle	= from_rgb (216, 191, 216)
  let tomato	= from_rgb (255, 99, 71)
  let turquoise	= from_rgb ( 64, 224, 208)
  let violet	= from_rgb (238, 130, 238)
  let wheat	= from_rgb (245, 222, 179)
  let white	= from_rgb (255, 255, 255)
  let whitesmoke	= from_rgb (245, 245, 245)
  let yellow	= from_rgb (255, 255, 0)
  let yellowgreen	= from_rgb (154, 205, 50)
end


module Combinator =
struct

  let gradient ~n left right =
    Pure.gradient ~n left right
    |> List.map Generator.color
  
  let aliceblue	= from_rgb (240, 248, 255) |> Generator.color 
  let antiquewhite	= from_rgb (250, 235, 215) |> Generator.color
  let aqua	= from_rgb ( 0, 255, 255) |> Generator.color
  let aquamarine	= from_rgb (127, 255, 212) |> Generator.color
  let azure	= from_rgb (240, 255, 255) |> Generator.color
  let beige	= from_rgb (245, 245, 220) |> Generator.color
  let bisque	= from_rgb (255, 228, 196) |> Generator.color
  let black	= from_rgb ( 0, 0, 0) |> Generator.color
  let blanchedalmond	= from_rgb (255, 235, 205) |> Generator.color
  let blue	= from_rgb ( 0, 0, 255) |> Generator.color
  let blueviolet	= from_rgb (138, 43, 226) |> Generator.color
  let brown	= from_rgb (165, 42, 42) |> Generator.color
  let burlywood	= from_rgb (222, 184, 135) |> Generator.color
  let cadetblue	= from_rgb ( 95, 158, 160) |> Generator.color
  let chartreuse	= from_rgb (127, 255, 0) |> Generator.color
  let chocolate	= from_rgb (210, 105, 30) |> Generator.color
  let coral	= from_rgb (255, 127, 80) |> Generator.color
  let cornflowerblue	= from_rgb (100, 149, 237) |> Generator.color
  let cornsilk	= from_rgb (255, 248, 220) |> Generator.color
  let crimson	= from_rgb (220, 20, 60) |> Generator.color
  let cyan	= from_rgb ( 0, 255, 255) |> Generator.color
  let darkblue	= from_rgb ( 0, 0, 139) |> Generator.color
  let darkcyan	= from_rgb ( 0, 139, 139) |> Generator.color
  let darkgoldenrod	= from_rgb (184, 134, 11) |> Generator.color
  let darkgray	= from_rgb (169, 169, 169) |> Generator.color
  let darkgreen	= from_rgb ( 0, 100, 0) |> Generator.color
  let darkgrey	= from_rgb (169, 169, 169) |> Generator.color
  let darkkhaki	= from_rgb (189, 183, 107) |> Generator.color
  let darkmagenta	= from_rgb (139, 0, 139) |> Generator.color
  let darkolivegreen	= from_rgb ( 85, 107, 47) |> Generator.color
  let darkorange	= from_rgb (255, 140, 0) |> Generator.color
  let darkorchid	= from_rgb (153, 50, 204) |> Generator.color
  let darkred	= from_rgb (139, 0, 0) |> Generator.color
  let darksalmon	= from_rgb (233, 150, 122) |> Generator.color
  let darkseagreen	= from_rgb (143, 188, 143) |> Generator.color
  let darkslateblue	= from_rgb ( 72, 61, 139) |> Generator.color
  let darkslategray	= from_rgb ( 47, 79, 79) |> Generator.color
  let darkslategrey	= from_rgb ( 47, 79, 79) |> Generator.color
  let darkturquoise	= from_rgb ( 0, 206, 209) |> Generator.color
  let darkviolet	= from_rgb (148, 0, 211) |> Generator.color
  let deeppink	= from_rgb (255, 20, 147) |> Generator.color
  let deepskyblue	= from_rgb ( 0, 191, 255) |> Generator.color
  let dimgray	= from_rgb (105, 105, 105) |> Generator.color
  let dimgrey	= from_rgb (105, 105, 105) |> Generator.color
  let dodgerblue	= from_rgb ( 30, 144, 255) |> Generator.color
  let firebrick	= from_rgb (178, 34, 34) |> Generator.color
  let floralwhite	= from_rgb (255, 250, 240) |> Generator.color
  let forestgreen	= from_rgb ( 34, 139, 34) |> Generator.color
  let fuchsia	= from_rgb (255, 0, 255) |> Generator.color
  let gainsboro	= from_rgb (220, 220, 220) |> Generator.color
  let ghostwhite	= from_rgb (248, 248, 255) |> Generator.color
  let gold	= from_rgb (255, 215, 0) |> Generator.color
  let goldenrod	= from_rgb (218, 165, 32) |> Generator.color
  let gray	= from_rgb (128, 128, 128) |> Generator.color
  let grey	= from_rgb (128, 128, 128) |> Generator.color
  let green	= from_rgb ( 0, 128, 0) |> Generator.color
  let greenyellow	= from_rgb (173, 255, 47) |> Generator.color
  let honeydew	= from_rgb (240, 255, 240) |> Generator.color
  let hotpink	= from_rgb (255, 105, 180) |> Generator.color
  let indianred	= from_rgb (205, 92, 92) |> Generator.color
  let indigo	= from_rgb ( 75, 0, 130) |> Generator.color
  let ivory	= from_rgb (255, 255, 240) |> Generator.color
  let khaki	= from_rgb (240, 230, 140) |> Generator.color
  let lavender	= from_rgb (230, 230, 250) |> Generator.color
  let lavenderblush	= from_rgb (255, 240, 245) |> Generator.color
  let lawngreen	= from_rgb (124, 252, 0) |> Generator.color
  let lemonchiffon	= from_rgb (255, 250, 205) |> Generator.color
  let lightblue	= from_rgb (173, 216, 230) |> Generator.color
  let lightcoral	= from_rgb (240, 128, 128) |> Generator.color
  let lightcyan	= from_rgb (224, 255, 255) |> Generator.color
  let lightgoldenrodyellow	= from_rgb (250, 250, 210) |> Generator.color
  let lightgray	= from_rgb (211, 211, 211) |> Generator.color
  let lightgreen	= from_rgb (144, 238, 144) |> Generator.color
  let lightgrey	= from_rgb (211, 211, 211) |> Generator.color
  let lightpink	= from_rgb (255, 182, 193) |> Generator.color
  let lightsalmon	= from_rgb (255, 160, 122) |> Generator.color
  let lightseagreen	= from_rgb ( 32, 178, 170) |> Generator.color
  let lightskyblue	= from_rgb (135, 206, 250) |> Generator.color
  let lightslategray	= from_rgb (119, 136, 153) |> Generator.color
  let lightslategrey	= from_rgb (119, 136, 153) |> Generator.color
  let lightsteelblue	= from_rgb (176, 196, 222) |> Generator.color
  let lightyellow	= from_rgb (255, 255, 224) |> Generator.color
  let lime	= from_rgb ( 0, 255, 0) |> Generator.color
  let limegreen	= from_rgb ( 50, 205, 50) |> Generator.color
  let linen	= from_rgb (250, 240, 230) |> Generator.color
  let magenta	= from_rgb (255, 0, 255) |> Generator.color
  let maroon	= from_rgb (128, 0, 0) |> Generator.color
  let mediumaquamarine	= from_rgb (102, 205, 170) |> Generator.color
  let mediumblue	= from_rgb ( 0, 0, 205) |> Generator.color
  let mediumorchid	= from_rgb (186, 85, 211) |> Generator.color
  let mediumpurple	= from_rgb (147, 112, 219) |> Generator.color
  let mediumseagreen	= from_rgb ( 60, 179, 113) |> Generator.color
  let mediumslateblue	= from_rgb (123, 104, 238) |> Generator.color
  let mediumspringgreen	= from_rgb ( 0, 250, 154) |> Generator.color
  let mediumturquoise	= from_rgb ( 72, 209, 204) |> Generator.color
  let mediumvioletred	= from_rgb (199, 21, 133) |> Generator.color
  let midnightblue	= from_rgb ( 25, 25, 112) |> Generator.color
  let mintcream	= from_rgb (245, 255, 250) |> Generator.color
  let mistyrose	= from_rgb (255, 228, 225) |> Generator.color
  let moccasin	= from_rgb (255, 228, 181) |> Generator.color
  let navajowhite	= from_rgb (255, 222, 173) |> Generator.color
  let navy	= from_rgb ( 0, 0, 128) |> Generator.color
  let oldlace	= from_rgb (253, 245, 230) |> Generator.color
  let olive	= from_rgb (128, 128, 0) |> Generator.color
  let olivedrab	= from_rgb (107, 142, 35) |> Generator.color
  let orange	= from_rgb (255, 165, 0) |> Generator.color
  let orangered	= from_rgb (255, 69, 0) |> Generator.color
  let orchid	= from_rgb (218, 112, 214) |> Generator.color
  let palegoldenrod	= from_rgb (238, 232, 170) |> Generator.color
  let palegreen	= from_rgb (152, 251, 152) |> Generator.color
  let paleturquoise	= from_rgb (175, 238, 238) |> Generator.color
  let palevioletred	= from_rgb (219, 112, 147) |> Generator.color
  let papayawhip	= from_rgb (255, 239, 213) |> Generator.color
  let peachpuff	= from_rgb (255, 218, 185) |> Generator.color
  let peru	= from_rgb (205, 133, 63) |> Generator.color
  let pink	= from_rgb (255, 192, 203) |> Generator.color
  let plum	= from_rgb (221, 160, 221) |> Generator.color
  let powderblue	= from_rgb (176, 224, 230) |> Generator.color
  let purple	= from_rgb (128, 0, 128) |> Generator.color
  let red	= from_rgb (255, 0, 0) |> Generator.color
  let rosybrown	= from_rgb (188, 143, 143) |> Generator.color
  let royalblue	= from_rgb ( 65, 105, 225) |> Generator.color
  let saddlebrown	= from_rgb (139, 69, 19) |> Generator.color
  let salmon	= from_rgb (250, 128, 114) |> Generator.color
  let sandybrown	= from_rgb (244, 164, 96) |> Generator.color
  let seagreen	= from_rgb ( 46, 139, 87) |> Generator.color
  let seashell	= from_rgb (255, 245, 238) |> Generator.color
  let sienna	= from_rgb (160, 82, 45) |> Generator.color
  let silver	= from_rgb (192, 192, 192) |> Generator.color
  let skyblue	= from_rgb (135, 206, 235) |> Generator.color
  let slateblue	= from_rgb (106, 90, 205) |> Generator.color
  let slategray	= from_rgb (112, 128, 144) |> Generator.color
  let slategrey	= from_rgb (112, 128, 144) |> Generator.color
  let snow	= from_rgb (255, 250, 250) |> Generator.color
  let springgreen	= from_rgb ( 0, 255, 127) |> Generator.color
  let steelblue	= from_rgb ( 70, 130, 180) |> Generator.color
  let tan	= from_rgb (210, 180, 140) |> Generator.color
  let teal	= from_rgb ( 0, 128, 128) |> Generator.color
  let thistle	= from_rgb (216, 191, 216) |> Generator.color
  let tomato	= from_rgb (255, 99, 71) |> Generator.color
  let turquoise	= from_rgb ( 64, 224, 208) |> Generator.color
  let violet	= from_rgb (238, 130, 238) |> Generator.color
  let wheat	= from_rgb (245, 222, 179) |> Generator.color
  let white	= from_rgb (255, 255, 255) |> Generator.color
  let whitesmoke	= from_rgb (245, 245, 245) |> Generator.color
  let yellow	= from_rgb (255, 255, 0) |> Generator.color
  let yellowgreen	= from_rgb (154, 205, 50) |> Generator.color
end
